@@ -1,3 +1,5 @@
+from __future__ import unicode_literals
+
 import json
 from traceback import format_exception_only
 
@@ -40,7 +42,7 @@ class AnymailError(Exception):
 
     def __str__(self):
         parts = [
-            " ".join([str(arg) for arg in self.args]),
+            " ".join([six.text_type(arg) for arg in self.args]),
             self.describe_raised_from(),
             self.describe_send(),
             self.describe_response(),
@@ -75,13 +77,13 @@ class AnymailError(Exception):
             except UnicodeDecodeError:
                 reason = reason.decode('iso-8859-1')
 
-        description = "%s API response %d: %s" % (self.esp_name or "ESP", self.status_code, reason)
+        description = "%s API response %d (%s)" % (self.esp_name or "ESP", self.status_code, reason)
         try:
             json_response = self.response.json()
-            description += "\n" + json.dumps(json_response, indent=2)
+            description += ":\n" + json.dumps(json_response, indent=2)
         except (AttributeError, KeyError, ValueError):  # not JSON = ValueError
             try:
-                description += " " + self.response.text
+                description += ": %r" % self.response.text
             except AttributeError:
                 pass
         return description
